use Time;

config param MAX_PRINT = 20;

/* INPUT PROCESSING */
writeln("\n2D Heat Diffusion program");
writef("Number of locales %i\n", numLocales);
writeln(" Enter number of X and Y data points:");
const Nx = stdin.read(int);
const Ny = stdin.read(int);
writeln(" Enter temperatures along edges:");
const Tedge = stdin.read(real);
writeln(" Enter maximum iterations:");
const Max_iter = stdin.read(int);
writeln(" Enter convergence:");
const converge = stdin.read(real);

const D = {0..Nx-1, 0..Ny-1};
const inner = {1..Nx-2,1..Ny-2};

writeln("-" * 72);
writeln();
writef(" Nx       %8i Ny     %8i \n", Nx, Ny);
writef(" Tedge    %8.2dr\n", Tedge);
writef(" Max_iter %8i\n", Max_iter);
writef(" Converge %8.2dr\n", converge);
writeln("-" * 72);

// Allocate data array
var told, tnew: [D] real;

/* *******Parallel Solution*********/
writeln("\n Begin Parallel Iterations");
told[inner] = 0.0;
told[0..#Nx, 0     ] = Tedge;
told[0..#Nx, Ny-1  ] = Tedge;
told[0,      0..#Ny] = Tedge;
told[Nx-1,   0..#Ny] = Tedge;
tnew = told;

var timer: Timer;
timer.start();

var iters = 0;
do {
  iters += 1;
  // update local values
  for (i,j) in inner do
    tnew[i,j] = 0.25*(told[i+1,j] + told[i-1,j]
                    + told[i,j+1] + told[i,j-1]);

  const mxdiff = max reduce abs (told[inner] - tnew[inner]);
  writef(" iteration %i convergence %dr\n", iters, mxdiff);
  told <=> tnew;
} while (mxdiff > converge && iters < Max_iter);

timer.stop();
writef("Total time parallel   %8.4dr\n", timer.elapsed());

/* Printout the result if small! */
if (Nx < MAX_PRINT && Ny < MAX_PRINT) {
  writef(" final grid values");
  for j in 0..#Ny {
    writef("\n Row %2i:   ",j);
    for i in 0..#Nx {
      writef("%10.2dr", told[i,j]);
      if i%5 == 4 then writef("\n           ");
    }
  }
}
writeln();

